<?php
/* --------------------------------------------------------------
 SetSessionParameters.php 2020-02-12
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2020 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

declare(strict_types=1);

namespace Gambio\Core\Application\Kernel\Bootstrapper;

use Gambio\Core\Application\ValueObjects\Path;
use Gambio\Core\Application\ValueObjects\Server;
use Gambio\Core\Application\ValueObjects\Url;
use Gambio\Core\Application\Application;
use Gambio\Core\Application\Kernel\AbstractBootstrapper;

/**
 * Class SetSessionParameters
 * @package Gambio\Core\Application\Kernel\Bootstrapper
 */
class SetSessionParameters extends AbstractBootstrapper
{
    /**
     * @inheritDoc
     */
    public function boot(Application $application): void
    {
        /**
         * @var Path   $path
         * @var Url    $url
         * @var Server $server
         */
        $path   = $application->get(Path::class);
        $url    = $application->get(Url::class);
        $server = $application->get(Server::class);
        
        $sessionStatus = \session_status();
        if ($sessionStatus === PHP_SESSION_DISABLED) {
            throw new \RuntimeException('PHP sessions MUST be enabled!');
        } elseif ($sessionStatus === PHP_SESSION_ACTIVE) {
            return;
        }
        
        @ini_set('session.gc_maxlifetime', defined('SESSION_GC_MAXLIFETIME') ? (string)SESSION_GC_MAXLIFETIME : '1440');
        @ini_set('session.gc_probability', defined('SESSION_GC_PROBABILITY') ? (string)SESSION_GC_PROBABILITY : '100');
        @ini_set('session.gc_divisor', defined('SESSION_GC_DIVISOR') ? (string)SESSION_GC_DIVISOR : '1000');
        @ini_set('session.use_trans_sid', '0');
        @ini_set('session.use_strict_mode', '1');
        @ini_set('session.use_only_cookies', '1');
        
        $shopHost      = parse_url($url->base(), PHP_URL_HOST);
        $shopHostParts = explode('.', $shopHost);
        if (count($shopHostParts) > 1 && $shopHostParts[0] === 'www') {
            unset($shopHostParts[0]);
        }
        $shopHost = implode('.', $shopHostParts);
        session_set_cookie_params(0, $url->path(), $shopHost, $server->sslEnabled(), true);
        
        $sessionName = $this->getSessionNamePostfix($url, $path);
        session_name('GXsid_' . $sessionName);
        
        if (defined('SESSION_SAVE_HANDLER') && defined('SESSION_SAVE_PATH')) {
            @ini_set('session.save_handler', SESSION_SAVE_HANDLER);
            session_save_path(SESSION_SAVE_PATH);
        } else {
            @ini_set('session.save_handler', 'files');
            session_save_path($path->base() . '/cache/sessions/');
        }
        
        // Todo: Fix user-space session handler
        $sessionHandlerClass = (defined('SESSION_USERSPACE_HANDLER')
                                && class_exists(SESSION_USERSPACE_HANDLER)) ? SESSION_USERSPACE_HANDLER : null;
        
        if (!empty($sessionHandlerClass)) {
            /** @var \SessionHandlerInterface $sessionHandler */ //$sessionHandler = MainFactory::create($sessionHandlerClass);
            //session_set_save_handler($sessionHandler, true);
        }
    }
    
    
    private function getSessionNamePostfix(Url $url, Path $path)
    {
        $urlHost  = $url->host();
        $urlPath  = "{$url->path()}/";
        $pathBase = "{$path->base()}/"; // stay compatible with code base below
        
        $identificationCriteria = [$urlHost, $urlPath, $pathBase];
        $postfixFileContent     = implode('|', $identificationCriteria);
        if (!is_writable($pathBase . 'media')) {
            throw new \RuntimeException('media directory MUST be writable!');
        }
        $sessionNamePostfixFiles = glob($pathBase . 'media/session_name_postfix_*', GLOB_NOSORT);
        if (empty($sessionNamePostfixFiles)) {
            $sessionNamePostfix = (string)substr(hash('sha256', (string)mt_rand()), 0, 16);
            file_put_contents($pathBase . 'media/session_name_postfix_' . $sessionNamePostfix, $postfixFileContent);
        } else { // count($sessionNamePostfixFiles) >= 1
            if (count($sessionNamePostfixFiles) > 1) {
                foreach (array_slice($sessionNamePostfixFiles, 1) as $extraFile) {
                    unlink($extraFile);
                }
            }
            $sessionNamePostfixFile     = $sessionNamePostfixFiles[0];
            $existingPostfixFileContent = file_get_contents($sessionNamePostfixFile);
            if ($existingPostfixFileContent === $postfixFileContent) {
                $sessionNamePostfix = (string)str_replace('session_name_postfix_',
                                                          '',
                                                          basename($sessionNamePostfixFile));
            } else {
                unlink($sessionNamePostfixFile);
                $sessionNamePostfix = (string)substr(hash('sha256', (string)mt_rand()), 0, 16);
                file_put_contents($pathBase . 'media/session_name_postfix_' . $sessionNamePostfix,
                                  $postfixFileContent);
            }
        }
        if (empty($sessionNamePostfix)) {
            throw new \RuntimeException('could not determine session name postfix');
        }
        
        return $sessionNamePostfix;
    }
}